package cz.vse.keg.patomat2;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;

import cz.vse.keg.patomat2.detection.OntologyPatternDetectionImpl;
import cz.vse.keg.patomat2.transformation.OntologyTransformation;
import cz.vse.keg.patomat2.transformation.OntologyTransformationImpl;
import cz.vse.keg.patomat2.transformation.OntologyTransformation.TransformationStrategy;
import cz.vse.keg.patomat2.transformation.RecursiveOntologyTransformation;
import cz.vse.keg.patomat2.transformation.pattern.InstructionGenerator;
import cz.vse.keg.patomat2.transformation.pattern.InstructionGeneratorImpl;
import cz.vse.keg.patomat2.transformation.pattern.TransformationPattern;
import cz.vse.keg.patomat2.transformation.pattern.TransformationPatternImpl;

import org.semanticweb.owlapi.model.OWLOntology;

/** 
 * TestLibrary class demonstrates pipeline of transformation where a tranformation pattern is applied recursively on a selected patten instance. 
 * This assumes installation of WordNet-3.0 at: * /usr/share/WordNet-3.0" and Stanford POS-tagger at "/usr/share/models". This code is provided within java library release.
 * @author Ondrej Zamazal
 */


public class TestLibrary6 {
	
	public static void main(String[] args) {
		//System.out.println(System.getProperty("user.home"));
		//String workingDirectory = args[0];
		String workingDirectory = "/home/ondrej";
		//String workingDirectory = "/C:/Users/person";
		//System.out.println(workingDirectory);
		//String tp1IRI="http://nb.vse.cz/~svabo/patomat/tp/new/tp_AR_classcentric1.xml"; //OK funguje
		String tp1IRI="http://nb.vse.cz/~svabo/patomat/tp/new/tp_AR_classcentric2.xml"; //OK funguje
		String ontology="http://nb.vse.cz/~svabo/patomat/ontology/confOf-imported2.owl";
		//String ontology="file:///home/ondrej/prace/PatOMat/TPs/ontology/confOf-imported2.owl";
		//Optional using part-of-speech tagger
		boolean POStagger=true;

		//preparatory phase: initialization
		OntologyTransformation<OWLOntology> transformation = new OntologyTransformationImpl(ontology,"/usr/share/WordNet-3.0", "/usr/share/models", workingDirectory);
		OntologyPatternDetectionImpl detection = new OntologyPatternDetectionImpl(transformation.getDictionaryPath(), transformation.getModelsPath(), workingDirectory);		
		
		//step 1 nonrecursive
		//preparatory phase: initialization
		TransformationPattern tp1 = new TransformationPatternImpl(tp1IRI);
		tp1.removeRecursiveAxiomsFromTP();
		//1st phase: detection
		//vystup z detekce pouzit v GUIPOTu pro vyber pattern instance uzivatelem
		ArrayList<String> bindingPlaceholders1 = detection.queryPatternNamingAspect2(tp1, ontology, true, false, POStagger); // == bez reasoningu
		
		//uzivatel v GUIPOTu vybral nejake pattern instances napr.
		//1. instance
		StringBuilder bindingPlaceholders1User = new StringBuilder();
		bindingPlaceholders1User.append("<pattern_instance>\n");
		bindingPlaceholders1User.append("<binding placeholder=\"?A\">Person</binding>\n");
		bindingPlaceholders1User.append("<binding placeholder=\"?B\">Participant</binding>\n");
		bindingPlaceholders1User.append("</pattern_instance>\n");
		
		//2. instance
		StringBuilder bindingPlaceholders2User = new StringBuilder();
		bindingPlaceholders2User.append("<pattern_instance>\n");
		bindingPlaceholders2User.append("<binding placeholder=\"?A\">Person</binding>\n");
		bindingPlaceholders2User.append("<binding placeholder=\"?B\">Assistant</binding>\n");
		bindingPlaceholders2User.append("</pattern_instance>\n");
		
		//3. instance
		StringBuilder bindingPlaceholders3User = new StringBuilder();
		bindingPlaceholders3User.append("<pattern_instance>\n");
		bindingPlaceholders3User.append("<binding placeholder=\"?A\">Person</binding>\n");
		bindingPlaceholders3User.append("<binding placeholder=\"?B\">Author</binding>\n");
		bindingPlaceholders3User.append("</pattern_instance>\n");
		
		//pripravit pattern instances pro vstup do transformace
		ArrayList<String> bindingPlaceholders = new ArrayList<String>();
		bindingPlaceholders.add(bindingPlaceholders1User.toString());
		bindingPlaceholders.add(bindingPlaceholders2User.toString());
		bindingPlaceholders.add(bindingPlaceholders3User.toString());
		
		//spustit rekurzivni transformaci
		RecursiveOntologyTransformation recursiveTransformation = new RecursiveOntologyTransformation(workingDirectory, ontology, tp1IRI, transformation);
		//Note: bindingPlaceholders are selected by a user
		recursiveTransformation.applyTransformationPatternRecursively(tp1, bindingPlaceholders);
	}
}
